@@ -1,21 +1,31 @@
 """
 Module with all the individual handlers, which execute git commands and return the results to the frontend.
 """
+import functools
 import json
 import os
 from pathlib import Path
-from tornado import web
+from typing import Tuple, Union
 
-from notebook.base.handlers import APIHandler
-from notebook.utils import url_path_join as ujoin, url2path
+import tornado
+from jupyter_server.base.handlers import APIHandler, path_regex
+from jupyter_server.services.contents.manager import ContentsManager
+from jupyter_server.utils import url2path, url_path_join
 from packaging.version import parse
 
-from ._version import __version__
-from .git import DEFAULT_REMOTE_NAME
+try:
+    import hybridcontents
+except ImportError:
+    hybridcontents = None
 
+from ._version import __version__
+from .git import DEFAULT_REMOTE_NAME, Git
+from .log import get_logger
 
 # Git configuration options exposed through the REST API
 ALLOWED_OPTIONS = ["user.name", "user.email"]
+# REST API namespace
+NAMESPACE = "/git"
 
 
 class GitHandler(APIHandler):
@@ -24,19 +34,37 @@ class GitHandler(APIHandler):
     """
 
     @property
-    def git(self):
+    def git(self) -> Git:
         return self.settings["git"]
 
+    @functools.lru_cache()
+    def url2localpath(
+        self, path: str, with_contents_manager: bool = False
+    ) -> Union[str, Tuple[str, ContentsManager]]:
+        """Get the local path from a JupyterLab server path.
+
+        Optionally it can also return the contents manager for that path.
+        """
+        cm = self.contents_manager
+
+        # Handle local manager of hybridcontents.HybridContentsManager
+        if hybridcontents is not None and isinstance(
+            cm, hybridcontents.HybridContentsManager
+        ):
+            _, cm, path = hybridcontents.hybridmanager._resolve_path(path, cm.managers)
+
+        local_path = os.path.join(os.path.expanduser(cm.root_dir), url2path(path))
+        return (local_path, cm) if with_contents_manager else local_path
+
 
 class GitCloneHandler(GitHandler):
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         Handler for the `git clone`
 
         Input format:
             {
-              'current_path': 'current_file_browser_path',
               'repo_url': 'https://github.com/path/to/myrepo',
               OPTIONAL 'auth': '{ 'username': '<username>',
                                   'password': '<password>'
@@ -45,7 +73,7 @@ class GitCloneHandler(GitHandler):
         """
         data = self.get_json_body()
         response = await self.git.clone(
-            data["current_path"], data["clone_url"], data.get("auth", None)
+            self.url2localpath(path), data["clone_url"], data.get("auth", None)
         )
 
         if response["code"] != 0:
@@ -63,24 +91,24 @@ class GitAllHistoryHandler(GitHandler):
     Called on refresh of extension's widget
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, calls individual handlers for
         'git show_top_level', 'git branch', 'git log', and 'git status'
         """
         body = self.get_json_body()
-        current_path = body["current_path"]
         history_count = body["history_count"]
+        local_path = self.url2localpath(path)
 
-        show_top_level = await self.git.show_top_level(current_path)
-        if show_top_level["code"] != 0:
+        show_top_level = await self.git.show_top_level(local_path)
+        if show_top_level.get("path") is None:
             self.set_status(500)
             self.finish(json.dumps(show_top_level))
         else:
-            branch = await self.git.branch(current_path)
-            log = await self.git.log(current_path, history_count)
-            status = await self.git.status(current_path)
+            branch = await self.git.branch(local_path)
+            log = await self.git.log(local_path, history_count)
+            status = await self.git.status(local_path)
 
             result = {
                 "code": show_top_level["code"],
@@ -100,13 +128,12 @@ class GitShowTopLevelHandler(GitHandler):
     Displays the git root directory inside a repository.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, displays the git root directory inside a repository.
         """
-        current_path = self.get_json_body()["current_path"]
-        result = await self.git.show_top_level(current_path)
+        result = await self.git.show_top_level(self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -120,14 +147,13 @@ class GitShowPrefixHandler(GitHandler):
     with respect to the root directory.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, displays the prefix path of a directory in a repository,
         with respect to the root directory.
         """
-        current_path = self.get_json_body()["current_path"]
-        result = await self.git.show_prefix(current_path)
+        result = await self.git.show_prefix(self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -139,13 +165,12 @@ class GitFetchHandler(GitHandler):
     Handler for 'git fetch'
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, fetch from remotes.
         """
-        current_path = self.get_json_body()["current_path"]
-        result = await self.git.fetch(current_path)
+        result = await self.git.fetch(self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -157,13 +182,12 @@ class GitStatusHandler(GitHandler):
     Handler for 'git status --porcelain', fetches the git status.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, fetches the git status.
         """
-        current_path = self.get_json_body()["current_path"]
-        result = await self.git.status(current_path)
+        result = await self.git.status(self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -176,16 +200,15 @@ class GitLogHandler(GitHandler):
     Fetches Commit SHA, Author Name, Commit Date & Commit Message.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler,
         fetches Commit SHA, Author Name, Commit Date & Commit Message.
         """
         body = self.get_json_body()
-        current_path = body["current_path"]
         history_count = body.get("history_count", 25)
-        result = await self.git.log(current_path, history_count)
+        result = await self.git.log(self.url2localpath(path), history_count)
 
         if result["code"] != 0:
             self.set_status(500)
@@ -199,16 +222,15 @@ class GitDetailedLogHandler(GitHandler):
     deletions in that commit.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, fetches file names of committed files, Number of
         insertions & deletions in that commit.
         """
         data = self.get_json_body()
         selected_hash = data["selected_hash"]
-        current_path = data["current_path"]
-        result = await self.git.detailed_log(selected_hash, current_path)
+        result = await self.git.detailed_log(selected_hash, self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -220,14 +242,13 @@ class GitDiffHandler(GitHandler):
     Handler for 'git diff --numstat'. Fetches changes between commits & working tree.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, fetches differences between commits & current working
         tree.
         """
-        top_repo_path = self.get_json_body()["top_repo_path"]
-        my_output = await self.git.diff(top_repo_path)
+        my_output = await self.git.diff(self.url2localpath(path))
 
         if my_output["code"] != 0:
             self.set_status(500)
@@ -239,13 +260,12 @@ class GitBranchHandler(GitHandler):
     Handler for 'git branch -a'. Fetches list of all branches in current repository
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, fetches all branches in current repository.
         """
-        current_path = self.get_json_body()["current_path"]
-        result = await self.git.branch(current_path)
+        result = await self.git.branch(self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -257,18 +277,19 @@ class GitBranchDeleteHandler(GitHandler):
     Handler for 'git branch -D <branch>'
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, delete branch in current repository.
 
+        Args:
+            path: Git repository path relatively to the server root
         Body: {
-            "current_path": Git repository path relatively to the server root,
             "branch": Branch name to be deleted
         }
         """
         data = self.get_json_body()
-        result = await self.git.branch_delete(data["current_path"], data["branch"])
+        result = await self.git.branch_delete(self.url2localpath(path), data["branch"])
 
         if result["code"] != 0:
             self.set_status(500)
@@ -283,18 +304,17 @@ class GitAddHandler(GitHandler):
     Adds one or all files to the staging area.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, adds one or all files into the staging area.
         """
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
         if data["add_all"]:
-            body = await self.git.add_all(top_repo_path)
+            body = await self.git.add_all(self.url2localpath(path))
         else:
             filename = data["filename"]
-            body = await self.git.add(filename, top_repo_path)
+            body = await self.git.add(filename, self.url2localpath(path))
 
         if body["code"] != 0:
             self.set_status(500)
@@ -307,12 +327,12 @@ class GitAddAllUnstagedHandler(GitHandler):
     untracked or staged files.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, adds all the changed files.
         """
-        body = await self.git.add_all_unstaged(self.get_json_body()["top_repo_path"])
+        body = await self.git.add_all_unstaged(self.url2localpath(path))
         if body["code"] != 0:
             self.set_status(500)
         self.finish(json.dumps(body))
@@ -324,12 +344,12 @@ class GitAddAllUntrackedHandler(GitHandler):
     untracked files, does not touch unstaged or staged files.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, adds all the untracked files.
         """
-        body = await self.git.add_all_untracked(self.get_json_body()["top_repo_path"])
+        body = await self.git.add_all_untracked(self.url2localpath(path))
         if body["code"] != 0:
             self.set_status(500)
         self.finish(json.dumps(body))
@@ -338,14 +358,13 @@ class GitAddAllUntrackedHandler(GitHandler):
 class GitRemoteAddHandler(GitHandler):
     """Handler for 'git remote add <name> <url>'."""
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """POST request handler to add a remote."""
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
         name = data.get("name", DEFAULT_REMOTE_NAME)
         url = data["url"]
-        output = await self.git.remote_add(top_repo_path, url, name)
+        output = await self.git.remote_add(self.url2localpath(path), url, name)
         if output["code"] == 0:
             self.set_status(201)
         else:
@@ -359,19 +378,19 @@ class GitResetHandler(GitHandler):
     Moves one or all files from the staged to the unstaged area.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler,
         moves one or all files from the staged to the unstaged area.
         """
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
+        local_path = self.url2localpath(path)
         if data["reset_all"]:
-            body = await self.git.reset_all(top_repo_path)
+            body = await self.git.reset_all(local_path)
         else:
             filename = data["filename"]
-            body = await self.git.reset(filename, top_repo_path)
+            body = await self.git.reset(filename, local_path)
 
         if body["code"] != 0:
             self.set_status(500)
@@ -384,12 +403,11 @@ class GitDeleteCommitHandler(GitHandler):
     Deletes the specified commit from the repository, leaving history intact.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
         commit_id = data["commit_id"]
-        body = await self.git.delete_commit(commit_id, top_repo_path)
+        body = await self.git.delete_commit(commit_id, self.url2localpath(path))
 
         if body["code"] != 0:
             self.set_status(500)
@@ -402,12 +420,11 @@ class GitResetToCommitHandler(GitHandler):
     Deletes all commits from head to the specified commit, making the specified commit the new head.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
         commit_id = data["commit_id"]
-        body = await self.git.reset_to_commit(commit_id, top_repo_path)
+        body = await self.git.reset_to_commit(commit_id, self.url2localpath(path))
 
         if body["code"] != 0:
             self.set_status(500)
@@ -419,24 +436,25 @@ class GitCheckoutHandler(GitHandler):
     Handler for 'git checkout <branchname>'. Changes the current working branch.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, changes between branches.
         """
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
+        local_path = self.url2localpath(path)
+
         if data["checkout_branch"]:
             if data["new_check"]:
                 body = await self.git.checkout_new_branch(
-                    data["branchname"], data["startpoint"], top_repo_path
+                    data["branchname"], data["startpoint"], local_path
                 )
             else:
-                body = await self.git.checkout_branch(data["branchname"], top_repo_path)
+                body = await self.git.checkout_branch(data["branchname"], local_path)
         elif data["checkout_all"]:
-            body = await self.git.checkout_all(top_repo_path)
+            body = await self.git.checkout_all(local_path)
         else:
-            body = await self.git.checkout(data["filename"], top_repo_path)
+            body = await self.git.checkout(data["filename"], local_path)
 
         if body["code"] != 0:
             self.set_status(500)
@@ -448,15 +466,14 @@ class GitCommitHandler(GitHandler):
     Handler for 'git commit -m <message>'. Commits files.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, commits files.
         """
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
         commit_msg = data["commit_msg"]
-        body = await self.git.commit(commit_msg, top_repo_path)
+        body = await self.git.commit(commit_msg, self.url2localpath(path))
 
         if body["code"] != 0:
             self.set_status(500)
@@ -464,20 +481,15 @@ class GitCommitHandler(GitHandler):
 
 
 class GitUpstreamHandler(GitHandler):
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         Handler for the `git rev-parse --abbrev-ref $CURRENT_BRANCH_NAME@{upstream}` on the repo. Used to check if there
         is a upstream branch defined for the current Git repo (and a side-effect is disabling the Git push/pull actions)
-
-        Input format:
-            {
-              'current_path': 'current_file_browser_path',
-            }
         """
-        current_path = self.get_json_body()["current_path"]
-        current_branch = await self.git.get_current_branch(current_path)
-        response = await self.git.get_upstream_branch(current_path, current_branch)
+        local_path = self.url2localpath(path)
+        current_branch = await self.git.get_current_branch(local_path)
+        response = await self.git.get_upstream_branch(local_path, current_branch)
         if response["code"] != 0:
             self.set_status(500)
         self.finish(json.dumps(response))
@@ -488,14 +500,14 @@ class GitPullHandler(GitHandler):
     Handler for 'git pull'. Pulls files from a remote branch.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, pulls files from a remote branch to your current branch.
         """
         data = self.get_json_body()
         response = await self.git.pull(
-            data["current_path"],
+            self.url2localpath(path),
             data.get("auth", None),
             data.get("cancel_on_conflict", False),
         )
@@ -512,27 +524,26 @@ class GitPushHandler(GitHandler):
     Pushes committed files to a remote branch.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler,
         pushes committed files from your current branch to a remote branch
 
         Request body:
         {
-            current_path: string, # Git repository path
             remote?: string # Remote to push to; i.e. <remote_name> or <remote_name>/<branch>
         }
         """
+        local_path = self.url2localpath(path)
         data = self.get_json_body()
-        current_path = data["current_path"]
         known_remote = data.get("remote")
 
-        current_local_branch = await self.git.get_current_branch(current_path)
+        current_local_branch = await self.git.get_current_branch(local_path)
 
         set_upstream = False
         current_upstream_branch = await self.git.get_upstream_branch(
-            current_path, current_local_branch
+            local_path, current_local_branch
         )
 
         if known_remote is not None:
@@ -551,7 +562,7 @@ class GitPushHandler(GitHandler):
             response = await self.git.push(
                 current_upstream_branch["remote_short_name"],
                 branch,
-                current_path,
+                local_path,
                 data.get("auth", None),
                 set_upstream,
             )
@@ -560,9 +571,9 @@ class GitPushHandler(GitHandler):
             # Allow users to specify upstream through their configuration
             # https://git-scm.com/docs/git-config#Documentation/git-config.txt-pushdefault
             # Or use the remote defined if only one remote exists
-            config = await self.git.config(current_path)
+            config = await self.git.config(local_path)
             config_options = config["options"]
-            list_remotes = await self.git.remote_show(current_path)
+            list_remotes = await self.git.remote_show(local_path)
             remotes = list_remotes.get("remotes", list())
             push_default = config_options.get("remote.pushdefault")
 
@@ -576,7 +587,7 @@ class GitPushHandler(GitHandler):
                 response = await self.git.push(
                     default_remote,
                     current_local_branch,
-                    current_path,
+                    local_path,
                     data.get("auth", None),
                     set_upstream=True,
                 )
@@ -600,13 +611,12 @@ class GitInitHandler(GitHandler):
     Handler for 'git init'. Initializes a repository.
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, initializes a repository.
         """
-        current_path = self.get_json_body()["current_path"]
-        body = await self.git.init(current_path)
+        body = await self.git.init(self.url2localpath(path))
 
         if body["code"] != 0:
             self.set_status(500)
@@ -615,9 +625,11 @@ class GitInitHandler(GitHandler):
 
 
 class GitChangedFilesHandler(GitHandler):
-    @web.authenticated
-    async def post(self):
-        body = await self.git.changed_files(**self.get_json_body())
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
+        body = await self.git.changed_files(
+            self.url2localpath(path), **self.get_json_body()
+        )
 
         if body["code"] != 0:
             self.set_status(500)
@@ -629,17 +641,16 @@ class GitConfigHandler(GitHandler):
     Handler for 'git config' commands
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST get (if no options are passed) or set configuration options
         """
-        data = self.get_json_body()
-        top_repo_path = data["path"]
+        data = self.get_json_body() or {}
         options = data.get("options", {})
 
         filtered_options = {k: v for k, v in options.items() if k in ALLOWED_OPTIONS}
-        response = await self.git.config(top_repo_path, **filtered_options)
+        response = await self.git.config(self.url2localpath(path), **filtered_options)
         if "options" in response:
             response["options"] = {
                 k: v for k, v in response["options"].items() if k in ALLOWED_OPTIONS
@@ -652,38 +663,62 @@ class GitConfigHandler(GitHandler):
         self.finish(json.dumps(response))
 
 
-class GitDiffContentHandler(GitHandler):
+class GitContentHandler(GitHandler):
     """
-    Handler for plain text diffs. Uses git show $REF:$FILE
-    Returns `prev_content` and `curr_content` with content of given file.
+    Handler to get file content at a certain git reference
     """
 
-    @web.authenticated
-    async def post(self):
-        cm = self.contents_manager
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         data = self.get_json_body()
         filename = data["filename"]
-        prev_ref = data["prev_ref"]
-        curr_ref = data["curr_ref"]
-        top_repo_path = os.path.join(cm.root_dir, url2path(data["top_repo_path"]))
-        response = await self.git.diff_content(
-            filename, prev_ref, curr_ref, top_repo_path
+        reference = data["reference"]
+        local_path, cm = self.url2localpath(path, with_contents_manager=True)
+        response = await self.git.get_content_at_reference(
+            filename, reference, local_path, cm
         )
         self.finish(json.dumps(response))
 
 
+class GitDiffNotebookHandler(GitHandler):
+    """
+    Returns nbdime diff of given notebook base content and remote content
+    """
+
+    @tornado.web.authenticated
+    async def post(self):
+        data = self.get_json_body()
+        try:
+            prev_content = data["previousContent"]
+            curr_content = data["currentContent"]
+        except KeyError as e:
+            get_logger().error(f"Missing key in POST request.", exc_info=e)
+            raise tornado.web.HTTPError(
+                status_code=400, reason=f"Missing POST key: {e}"
+            )
+        try:
+            content = await self.git.get_nbdiff(prev_content, curr_content)
+        except Exception as e:
+            get_logger().error(f"Error computing notebook diff.", exc_info=e)
+            raise tornado.web.HTTPError(
+                status_code=500,
+                reason=f"Error diffing content: {e}.",
+            ) from e
+        self.finish(json.dumps(content))
+
+
 class GitIgnoreHandler(GitHandler):
     """
     Handler to manage .gitignore
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST add entry in .gitignore
         """
+        local_path = self.url2localpath(path)
         data = self.get_json_body()
-        top_repo_path = data["top_repo_path"]
         file_path = data.get("file_path", None)
         use_extension = data.get("use_extension", False)
         if file_path:
@@ -691,9 +726,9 @@ class GitIgnoreHandler(GitHandler):
                 suffixes = Path(file_path).suffixes
                 if len(suffixes) > 0:
                     file_path = "**/*" + ".".join(suffixes)
-            body = await self.git.ignore(top_repo_path, file_path)
+            body = await self.git.ignore(local_path, file_path)
         else:
-            body = await self.git.ensure_gitignore(top_repo_path)
+            body = await self.git.ensure_gitignore(local_path)
 
         if body["code"] != 0:
             self.set_status(500)
@@ -701,7 +736,7 @@ class GitIgnoreHandler(GitHandler):
 
 
 class GitSettingsHandler(GitHandler):
-    @web.authenticated
+    @tornado.web.authenticated
     async def get(self):
         jlab_version = self.get_query_argument("version", None)
         if jlab_version is not None:
@@ -713,16 +748,13 @@ class GitSettingsHandler(GitHandler):
             self.log.debug(
                 "[jupyterlab_git] Failed to execute 'git' command: {!s}".format(error)
             )
-        server_version = str(parse(__version__))
-        # Similar to https://github.com/jupyter/nbdime/blob/master/nbdime/webapp/nb_server_extension.py#L90-L91
-        root_dir = getattr(self.contents_manager, "root_dir", None)
-        server_root = None if root_dir is None else Path(root_dir).as_posix()
+        server_version = str(__version__)
+
         self.finish(
             json.dumps(
                 {
                     "frontendVersion": jlab_version,
                     "gitVersion": git_version,
-                    "serverRoot": server_root,
                     "serverVersion": server_version,
                 }
             )
@@ -734,13 +766,12 @@ class GitTagHandler(GitHandler):
     Handler for 'git tag '. Fetches list of all tags in current repository
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, fetches all tags in current repository.
         """
-        current_path = self.get_json_body()["current_path"]
-        result = await self.git.tags(current_path)
+        result = await self.git.tags(self.url2localpath(path))
 
         if result["code"] != 0:
             self.set_status(500)
@@ -752,75 +783,72 @@ class GitTagCheckoutHandler(GitHandler):
     Handler for 'git tag checkout '. Checkout the tag version of repo
     """
 
-    @web.authenticated
-    async def post(self):
+    @tornado.web.authenticated
+    async def post(self, path: str = ""):
         """
         POST request handler, checkout the tag version to a branch.
         """
         data = self.get_json_body()
-        current_path = data["current_path"]
         tag = data["tag_id"]
-        result = await self.git.tag_checkout(current_path, tag)
+        result = await self.git.tag_checkout(self.url2localpath(path), tag)
 
         if result["code"] != 0:
             self.set_status(500)
         self.finish(json.dumps(result))
 
 
-# FIXME remove for 0.22 release - this avoid error when upgrading from 0.20 to 0.21 if the frontend
-# has not been rebuilt yet.
-class GitServerRootHandler(GitHandler):
-    @web.authenticated
-    async def get(self):
-        # Similar to https://github.com/jupyter/nbdime/blob/master/nbdime/webapp/nb_server_extension.py#L90-L91
-        root_dir = getattr(self.contents_manager, "root_dir", None)
-        server_root = None if root_dir is None else Path(root_dir).as_posix()
-        self.finish(json.dumps({"server_root": server_root}))
-
-
 def setup_handlers(web_app):
     """
     Setups all of the git command handlers.
     Every handler is defined here, to be used in git.py file.
     """
 
-    git_handlers = [
-        ("/git/add", GitAddHandler),
-        ("/git/add_all_unstaged", GitAddAllUnstagedHandler),
-        ("/git/add_all_untracked", GitAddAllUntrackedHandler),
-        ("/git/all_history", GitAllHistoryHandler),
-        ("/git/branch", GitBranchHandler),
-        ("/git/branch/delete", GitBranchDeleteHandler),
-        ("/git/changed_files", GitChangedFilesHandler),
-        ("/git/checkout", GitCheckoutHandler),
-        ("/git/clone", GitCloneHandler),
-        ("/git/commit", GitCommitHandler),
-        ("/git/config", GitConfigHandler),
-        ("/git/delete_commit", GitDeleteCommitHandler),
-        ("/git/detailed_log", GitDetailedLogHandler),
-        ("/git/diff", GitDiffHandler),
-        ("/git/diffcontent", GitDiffContentHandler),
-        ("/git/init", GitInitHandler),
-        ("/git/log", GitLogHandler),
-        ("/git/pull", GitPullHandler),
-        ("/git/push", GitPushHandler),
-        ("/git/remote/add", GitRemoteAddHandler),
-        ("/git/remote/fetch", GitFetchHandler),
-        ("/git/reset", GitResetHandler),
-        ("/git/reset_to_commit", GitResetToCommitHandler),
-        ("/git/server_root", GitServerRootHandler),
-        ("/git/settings", GitSettingsHandler),
-        ("/git/show_prefix", GitShowPrefixHandler),
-        ("/git/show_top_level", GitShowTopLevelHandler),
-        ("/git/status", GitStatusHandler),
-        ("/git/upstream", GitUpstreamHandler),
-        ("/git/ignore", GitIgnoreHandler),
-        ("/git/tags", GitTagHandler),
-        ("/git/tag_checkout", GitTagCheckoutHandler),
+    handlers_with_path = [
+        ("/add_all_unstaged", GitAddAllUnstagedHandler),
+        ("/add_all_untracked", GitAddAllUntrackedHandler),
+        ("/all_history", GitAllHistoryHandler),
+        ("/branch/delete", GitBranchDeleteHandler),
+        ("/branch", GitBranchHandler),
+        ("/changed_files", GitChangedFilesHandler),
+        ("/checkout", GitCheckoutHandler),
+        ("/clone", GitCloneHandler),
+        ("/commit", GitCommitHandler),
+        ("/config", GitConfigHandler),
+        ("/content", GitContentHandler),
+        ("/delete_commit", GitDeleteCommitHandler),
+        ("/detailed_log", GitDetailedLogHandler),
+        ("/diff", GitDiffHandler),
+        ("/init", GitInitHandler),
+        ("/log", GitLogHandler),
+        ("/pull", GitPullHandler),
+        ("/push", GitPushHandler),
+        ("/remote/add", GitRemoteAddHandler),
+        ("/remote/fetch", GitFetchHandler),
+        ("/reset", GitResetHandler),
+        ("/reset_to_commit", GitResetToCommitHandler),
+        ("/show_prefix", GitShowPrefixHandler),
+        ("/show_top_level", GitShowTopLevelHandler),
+        ("/status", GitStatusHandler),
+        ("/upstream", GitUpstreamHandler),
+        ("/ignore", GitIgnoreHandler),
+        ("/tags", GitTagHandler),
+        ("/tag_checkout", GitTagCheckoutHandler),
+        ("/add", GitAddHandler),
+    ]
+
+    handlers = [
+        ("/diffnotebook", GitDiffNotebookHandler),
+        ("/settings", GitSettingsHandler),
     ]
 
     # add the baseurl to our paths
     base_url = web_app.settings["base_url"]
-    git_handlers = [(ujoin(base_url, x[0]), x[1]) for x in git_handlers]
+    git_handlers = [
+        (url_path_join(base_url, NAMESPACE + path_regex + endpoint), handler)
+        for endpoint, handler in handlers_with_path
+    ] + [
+        (url_path_join(base_url, NAMESPACE + endpoint), handler)
+        for endpoint, handler in handlers
+    ]
 
     web_app.add_handlers(".*", git_handlers)
