@@ -1,18 +1,16 @@
-# python lib
-import os
-from unittest.mock import Mock, call, patch
+from pathlib import Path
+from unittest.mock import call, patch
 
 import pytest
-import tornado
 
-# local lib
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, maybe_future
+from .testutils import maybe_future
 
 
-def test_is_remote_branch():
-    test_cases = [
+@pytest.mark.parametrize(
+    "branch,expected",
+    [
         ("refs/heads/feature-foo", False),
         ("refs/heads/master", False),
         ("refs/remotes/origin/feature-foo", True),
@@ -20,12 +18,11 @@ def test_is_remote_branch():
         ("refs/stash", False),
         ("refs/tags/v0.1.0", False),
         ("refs/tags/blah@0.2.0", False),
-    ]
-    for test_case in test_cases:
-        actual_response = Git(FakeContentManager("/bin"))._is_remote_branch(
-            test_case[0]
-        )
-        assert test_case[1] == actual_response
+    ],
+)
+def test_is_remote_branch(branch, expected):
+    actual_response = Git()._is_remote_branch(branch)
+    assert expected == actual_response
 
 
 @pytest.mark.asyncio
@@ -37,15 +34,13 @@ async def test_get_current_branch_success():
 
         # When
         actual_response = await (
-            Git(FakeContentManager("/bin")).get_current_branch(
-                current_path="test_curr_path"
-            )
+            Git().get_current_branch(path=str(Path("/bin/test_curr_path")))
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "symbolic-ref", "--short", "HEAD"],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert "feature-foo" == actual_response
 
@@ -53,7 +48,7 @@ async def test_get_current_branch_success():
 @pytest.mark.asyncio
 async def test_checkout_branch_noref_success():
     branch = "test-branch"
-    curr_path = "test_curr_path"
+    curr_path = str(Path("/bin/test_curr_path"))
     stdout_message = "checkout output from git"
     stderr_message = ""
     rc = 0
@@ -68,8 +63,8 @@ async def test_checkout_branch_noref_success():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).checkout_branch(
-                branchname=branch, current_path=curr_path
+            actual_response = await Git().checkout_branch(
+                branchname=branch, path=curr_path
             )
 
             # Then
@@ -77,7 +72,8 @@ async def test_checkout_branch_noref_success():
 
             cmd = ["git", "checkout", branch]
             mock_execute.assert_called_once_with(
-                cmd, cwd=os.path.join("/bin", "test_curr_path")
+                cmd,
+                cwd=str(Path("/bin") / "test_curr_path"),
             )
 
             assert {"code": rc, "message": stdout_message} == actual_response
@@ -86,7 +82,7 @@ async def test_checkout_branch_noref_success():
 @pytest.mark.asyncio
 async def test_checkout_branch_noref_failure():
     branch = "test-branch"
-    curr_path = "test_curr_path"
+    curr_path = str(Path("/bin/test_curr_path"))
     stdout_message = ""
     stderr_message = (
         "error: pathspec '{}' did not match any file(s) known to git".format(branch)
@@ -102,8 +98,8 @@ async def test_checkout_branch_noref_failure():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).checkout_branch(
-                branchname=branch, current_path=curr_path
+            actual_response = await Git().checkout_branch(
+                branchname=branch, path=curr_path
             )
 
             # Then
@@ -111,7 +107,8 @@ async def test_checkout_branch_noref_failure():
 
             cmd = ["git", "checkout", branch]
             mock_execute.assert_called_once_with(
-                cmd, cwd=os.path.join("/bin", "test_curr_path")
+                cmd,
+                cwd=str(Path("/bin") / "test_curr_path"),
             )
 
             assert {
@@ -124,7 +121,7 @@ async def test_checkout_branch_noref_failure():
 @pytest.mark.asyncio
 async def test_checkout_branch_remoteref_success():
     branch = "test-branch"
-    curr_path = "test_curr_path"
+    curr_path = str(Path("/bin/test_curr_path"))
     stdout_message = "checkout output from git"
     stderr_message = ""
     rc = 0
@@ -141,8 +138,8 @@ async def test_checkout_branch_remoteref_success():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).checkout_branch(
-                branchname=branch, current_path=curr_path
+            actual_response = await Git().checkout_branch(
+                branchname=branch, path=curr_path
             )
 
             # Then
@@ -150,7 +147,8 @@ async def test_checkout_branch_remoteref_success():
 
             cmd = ["git", "checkout", "--track", branch]
             mock_execute.assert_called_once_with(
-                cmd, cwd=os.path.join("/bin", "test_curr_path")
+                cmd,
+                cwd=str(Path("/bin") / "test_curr_path"),
             )
 
             assert {"code": rc, "message": stdout_message} == actual_response
@@ -159,7 +157,7 @@ async def test_checkout_branch_remoteref_success():
 @pytest.mark.asyncio
 async def test_checkout_branch_headsref_failure():
     branch = "test-branch"
-    curr_path = "test_curr_path"
+    curr_path = str(Path("/bin/test_curr_path"))
     stdout_message = ""
     stderr_message = (
         "error: pathspec '{}' did not match any file(s) known to git".format(branch)
@@ -178,8 +176,8 @@ async def test_checkout_branch_headsref_failure():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).checkout_branch(
-                branchname=branch, current_path=curr_path
+            actual_response = await Git().checkout_branch(
+                branchname=branch, path=curr_path
             )
 
             # Then
@@ -187,7 +185,8 @@ async def test_checkout_branch_headsref_failure():
 
             cmd = ["git", "checkout", branch]
             mock_execute.assert_called_once_with(
-                cmd, cwd=os.path.join("/bin", "test_curr_path")
+                cmd,
+                cwd=str(Path("/bin") / "test_curr_path"),
             )
             assert {
                 "code": rc,
@@ -215,14 +214,15 @@ async def test_checkout_branch_headsref_success():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).checkout_branch(
-                branchname=branch, current_path="test_curr_path"
+            actual_response = await Git().checkout_branch(
+                branchname=branch, path=str(Path("/bin/test_curr_path"))
             )
 
             # Then
             cmd = ["git", "checkout", branch]
             mock_execute.assert_called_once_with(
-                cmd, cwd=os.path.join("/bin", "test_curr_path")
+                cmd,
+                cwd=str(Path("/bin") / "test_curr_path"),
             )
             assert {"code": rc, "message": stdout_message} == actual_response
 
@@ -248,14 +248,15 @@ async def test_checkout_branch_remoteref_failure():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).checkout_branch(
-                branchname=branch, current_path="test_curr_path"
+            actual_response = await Git().checkout_branch(
+                branchname=branch, path=str(Path("/bin/test_curr_path"))
             )
 
             # Then
             cmd = ["git", "checkout", "--track", branch]
             mock_execute.assert_called_once_with(
-                cmd, cwd=os.path.join("/bin", "test_curr_path")
+                cmd,
+                cwd=str(Path("/bin") / "test_curr_path"),
             )
             assert {
                 "code": rc,
@@ -276,14 +277,14 @@ async def test_get_branch_reference_success():
         mock_execute.return_value = maybe_future((0, reference, ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin"))._get_branch_reference(
-            branchname=branch, current_path="test_curr_path"
+        actual_response = await Git()._get_branch_reference(
+            branchname=branch, path=str(Path("/bin/test_curr_path"))
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "rev-parse", "--symbolic-full-name", branch],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert actual_response == reference
 
@@ -306,14 +307,14 @@ async def test_get_branch_reference_failure():
         )
 
         # When
-        actual_response = await Git(FakeContentManager("/bin"))._get_branch_reference(
-            branchname=branch, current_path="test_curr_path"
+        actual_response = await Git()._get_branch_reference(
+            branchname=branch, path=str(Path("/bin/test_curr_path"))
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "rev-parse", "--symbolic-full-name", branch],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert actual_response is None
 
@@ -332,14 +333,12 @@ async def test_get_current_branch_failure():
 
         # When
         with pytest.raises(Exception) as error:
-            await Git(FakeContentManager("/bin")).get_current_branch(
-                current_path="test_curr_path"
-            )
+            await Git().get_current_branch(path=str(Path("/bin/test_curr_path")))
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "symbolic-ref", "--short", "HEAD"],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert (
             "Error [fatal: Not a git repository (or any of the parent directories): .git] "
@@ -361,13 +360,14 @@ async def test_get_current_branch_detached_success():
         mock_execute.return_value = maybe_future((0, "\n".join(process_output), ""))
 
         # When
-        actual_response = await Git(
-            FakeContentManager("/bin")
-        )._get_current_branch_detached(current_path="test_curr_path")
+        actual_response = await Git()._get_current_branch_detached(
+            path=str(Path("/bin/test_curr_path"))
+        )
 
         # Then
         mock_execute.assert_called_once_with(
-            ["git", "branch", "-a"], cwd=os.path.join("/bin", "test_curr_path")
+            ["git", "branch", "-a"],
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert "(HEAD detached at origin/feature-foo)" == actual_response
 
@@ -386,13 +386,14 @@ async def test_get_current_branch_detached_failure():
 
         # When
         with pytest.raises(Exception) as error:
-            await Git(FakeContentManager("/bin"))._get_current_branch_detached(
-                current_path="test_curr_path"
+            await Git()._get_current_branch_detached(
+                path=str(Path("/bin/test_curr_path"))
             )
 
         # Then
         mock_execute.assert_called_once_with(
-            ["git", "branch", "-a"], cwd=os.path.join("/bin", "test_curr_path")
+            ["git", "branch", "-a"],
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert (
             "Error [fatal: Not a git repository (or any of the parent directories): .git] "
@@ -408,6 +409,8 @@ async def test_get_current_branch_detached_failure():
         ("feature-foo", "master", "origin/withslash"),
         ("master", "master", "origin"),
         ("feature/bar", "feature-foo", ""),
+        # Test upstream branch name starts with a letter contained in remote name
+        ("rbranch", "rbranch", "origin"),
     ],
 )
 async def test_get_upstream_branch_success(branch, upstream, remotename):
@@ -419,8 +422,8 @@ async def test_get_upstream_branch_success(branch, upstream, remotename):
         ]
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).get_upstream_branch(
-            current_path="test_curr_path", branch_name=branch
+        actual_response = await Git().get_upstream_branch(
+            path=str(Path("/bin/test_curr_path")), branch_name=branch
         )
 
         # Then
@@ -433,11 +436,11 @@ async def test_get_upstream_branch_success(branch, upstream, remotename):
                         "--abbrev-ref",
                         "{}@{{upstream}}".format(branch),
                     ],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
                 call(
                     ["git", "config", "--local", "branch.{}.remote".format(branch)],
-                    cwd="/bin/test_curr_path",
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
             ],
             any_order=False,
@@ -475,8 +478,8 @@ async def test_get_upstream_branch_failure(outputs, message):
         mock_execute.return_value = maybe_future(outputs)
 
         # When
-        response = await Git(FakeContentManager("/bin")).get_upstream_branch(
-            current_path="test_curr_path", branch_name="blah"
+        response = await Git().get_upstream_branch(
+            path=str(Path("/bin/test_curr_path")), branch_name="blah"
         )
         expected = {
             "code": 128,
@@ -491,7 +494,7 @@ async def test_get_upstream_branch_failure(outputs, message):
             [
                 call(
                     ["git", "rev-parse", "--abbrev-ref", "blah@{upstream}"],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 )
             ],
             any_order=False,
@@ -505,15 +508,15 @@ async def test_get_tag_success():
         mock_execute.return_value = maybe_future((0, "v0.3.0", ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin"))._get_tag(
-            current_path="test_curr_path",
+        actual_response = await Git()._get_tag(
+            path=str(Path("/bin/test_curr_path")),
             commit_sha="abcdefghijklmnopqrstuvwxyz01234567890123",
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "describe", "--tags", "abcdefghijklmnopqrstuvwxyz01234567890123"],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
         assert "v0.3.0" == actual_response
 
@@ -535,8 +538,8 @@ async def test_get_tag_failure():
 
         # When
         with pytest.raises(Exception) as error:
-            await Git(FakeContentManager("/bin"))._get_tag(
-                current_path="test_curr_path", commit_sha="blah"
+            await Git()._get_tag(
+                path=str(Path("/bin/test_curr_path")), commit_sha="blah"
             )
 
         assert (
@@ -545,8 +548,8 @@ async def test_get_tag_failure():
             == str(error.value)
         )
 
-        actual_response = await Git(FakeContentManager("/bin"))._get_tag(
-            current_path="test_curr_path",
+        actual_response = await Git()._get_tag(
+            path=str(Path("/bin/test_curr_path")),
             commit_sha="01234567899999abcdefghijklmnopqrstuvwxyz",
         )
 
@@ -557,7 +560,7 @@ async def test_get_tag_failure():
             [
                 call(
                     ["git", "describe", "--tags", "blah"],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
                 call(
                     [
@@ -566,7 +569,7 @@ async def test_get_tag_failure():
                         "--tags",
                         "01234567899999abcdefghijklmnopqrstuvwxyz",
                     ],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
             ],
             any_order=False,
@@ -582,7 +585,7 @@ async def test_no_tags():
         )
 
         # When
-        actual_response = await Git(FakeContentManager("/bin"))._get_tag(
+        actual_response = await Git()._get_tag(
             "/path/foo", "768c79ad661598889f29bdf8916f4cc488f5062a"
         )
 
@@ -670,9 +673,7 @@ async def test_branch_success():
         }
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).branch(
-            current_path="test_curr_path"
-        )
+        actual_response = await Git().branch(path=str(Path("/bin/test_curr_path")))
 
         # Then
         mock_execute.assert_has_calls(
@@ -685,7 +686,7 @@ async def test_branch_success():
                         "--format=%(refname:short)%09%(objectname)%09%(upstream:short)%09%(HEAD)",
                         "refs/heads/",
                     ],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
                 # call to get refs/remotes
                 call(
@@ -695,7 +696,7 @@ async def test_branch_success():
                         "--format=%(refname:short)%09%(objectname)",
                         "refs/remotes/",
                     ],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
             ],
             any_order=False,
@@ -728,13 +729,12 @@ async def test_branch_failure():
         }
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).branch(
-            current_path="test_curr_path"
-        )
+        actual_response = await Git().branch(path=str(Path("/bin/test_curr_path")))
 
         # Then
         mock_execute.assert_called_once_with(
-            expected_cmd, cwd=os.path.join("/bin", "test_curr_path")
+            expected_cmd,
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
 
         assert expected_response == actual_response
@@ -806,9 +806,7 @@ async def test_branch_success_detached_head():
         }
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).branch(
-            current_path="test_curr_path"
-        )
+        actual_response = await Git().branch(path=str(Path("/bin/test_curr_path")))
 
         # Then
         mock_execute.assert_has_calls(
@@ -821,16 +819,17 @@ async def test_branch_success_detached_head():
                         "--format=%(refname:short)%09%(objectname)%09%(upstream:short)%09%(HEAD)",
                         "refs/heads/",
                     ],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
                 # call to get current branch
                 call(
                     ["git", "symbolic-ref", "--short", "HEAD"],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
                 # call to get current branch name given a detached head
                 call(
-                    ["git", "branch", "-a"], cwd=os.path.join("/bin", "test_curr_path")
+                    ["git", "branch", "-a"],
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
                 # call to get refs/remotes
                 call(
@@ -840,7 +839,7 @@ async def test_branch_success_detached_head():
                         "--format=%(refname:short)%09%(objectname)",
                         "refs/remotes/",
                     ],
-                    cwd=os.path.join("/bin", "test_curr_path"),
+                    cwd=str(Path("/bin") / "test_curr_path"),
                 ),
             ],
             any_order=False,
