@@ -1,12 +1,11 @@
-import os
-from unittest.mock import Mock, call, patch
+from pathlib import Path
+from unittest.mock import patch
 
 import pytest
-import tornado
 
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, maybe_future
+from .testutils import maybe_future
 
 
 @pytest.mark.asyncio
@@ -18,14 +17,14 @@ async def test_git_clone_success():
             mock_execute.return_value = maybe_future((0, output, "error"))
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).clone(
-                current_path="test_curr_path", repo_url="ghjkhjkl"
+            actual_response = await Git().clone(
+                path=str(Path("/bin/test_curr_path")), repo_url="ghjkhjkl"
             )
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "clone", "ghjkhjkl"],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd=str(Path("/bin") / "test_curr_path"),
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
             )
             assert {"code": 0, "message": output} == actual_response
@@ -46,14 +45,14 @@ async def test_git_clone_failure_from_git():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).clone(
-                current_path="test_curr_path", repo_url="ghjkhjkl"
+            actual_response = await Git().clone(
+                path=str(Path("/bin/test_curr_path")), repo_url="ghjkhjkl"
             )
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "clone", "ghjkhjkl"],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd=str(Path("/bin") / "test_curr_path"),
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
             )
             assert {
@@ -72,8 +71,8 @@ async def test_git_clone_with_auth_success():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).clone(
-                current_path="test_curr_path", repo_url="ghjkhjkl", auth=auth
+            actual_response = await Git().clone(
+                path=str(Path("/bin/test_curr_path")), repo_url="ghjkhjkl", auth=auth
             )
 
             # Then
@@ -81,7 +80,7 @@ async def test_git_clone_with_auth_success():
                 ["git", "clone", "ghjkhjkl", "-q"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd=str(Path("/bin") / "test_curr_path"),
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {"code": 0, "message": output} == actual_response
@@ -103,8 +102,8 @@ async def test_git_clone_with_auth_wrong_repo_url_failure_from_git():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).clone(
-                current_path="test_curr_path", repo_url="ghjkhjkl", auth=auth
+            actual_response = await Git().clone(
+                path=str(Path("/bin/test_curr_path")), repo_url="ghjkhjkl", auth=auth
             )
 
             # Then
@@ -112,7 +111,7 @@ async def test_git_clone_with_auth_wrong_repo_url_failure_from_git():
                 ["git", "clone", "ghjkhjkl", "-q"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd=str(Path("/bin") / "test_curr_path"),
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {
@@ -141,8 +140,8 @@ async def test_git_clone_with_auth_auth_failure_from_git():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).clone(
-                current_path="test_curr_path", repo_url="ghjkhjkl", auth=auth
+            actual_response = await Git().clone(
+                path=str(Path("/bin/test_curr_path")), repo_url="ghjkhjkl", auth=auth
             )
 
             # Then
@@ -150,7 +149,7 @@ async def test_git_clone_with_auth_auth_failure_from_git():
                 ["git", "clone", "ghjkhjkl", "-q"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd=str(Path("/bin") / "test_curr_path"),
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {
