@@ -1,14 +1,11 @@
-# python lib
-import os
-from unittest.mock import Mock, call, patch
+from pathlib import Path
+from unittest.mock import patch
 
 import pytest
-import tornado
 
-# local lib
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, maybe_future
+from .testutils import maybe_future
 
 
 @pytest.mark.asyncio
@@ -92,9 +89,9 @@ async def test_detailed_log():
         }
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).detailed_log(
+        actual_response = await Git().detailed_log(
             selected_hash="f29660a2472e24164906af8653babeb48e4bf2ab",
-            current_path="test_curr_path",
+            path=str(Path("/bin/test_curr_path")),
         )
 
         # Then
@@ -108,7 +105,7 @@ async def test_detailed_log():
                 "-z",
                 "f29660a2472e24164906af8653babeb48e4bf2ab",
             ],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd=str(Path("/bin") / "test_curr_path"),
         )
 
         assert expected_response == actual_response
