@@ -1,20 +1,22 @@
-import os
+import json
+import nbformat
+from pathlib import Path
 from subprocess import CalledProcessError
-from unittest.mock import Mock, call, patch
+from unittest.mock import patch
 
 import pytest
 import tornado
 
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, maybe_future
+from .testutils import maybe_future
 
 
 @pytest.mark.asyncio
 async def test_changed_files_invalid_input():
     with pytest.raises(tornado.web.HTTPError):
-        await Git(FakeContentManager("/bin")).changed_files(
-            current_path="test-path", base="64950a634cd11d1a01ddfedaeffed67b531cb11e"
+        await Git().changed_files(
+            path="test-path", base="64950a634cd11d1a01ddfedaeffed67b531cb11e"
         )
 
 
@@ -26,8 +28,8 @@ async def test_changed_files_single_commit():
         mock_execute.return_value = maybe_future((0, "file1.ipynb\x00file2.py\x00", ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).changed_files(
-            current_path="test-path",
+        actual_response = await Git().changed_files(
+            path="test-path",
             single_commit="64950a634cd11d1a01ddfedaeffed67b531cb11e^!",
         )
 
@@ -40,7 +42,7 @@ async def test_changed_files_single_commit():
                 "--name-only",
                 "-z",
             ],
-            cwd="/bin/test-path",
+            cwd="test-path",
         )
         assert {"code": 0, "files": ["file1.ipynb", "file2.py"]} == actual_response
 
@@ -52,13 +54,14 @@ async def test_changed_files_working_tree():
         mock_execute.return_value = maybe_future((0, "file1.ipynb\x00file2.py", ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).changed_files(
-            current_path="test-path", base="WORKING", remote="HEAD"
+        actual_response = await Git().changed_files(
+            path="test-path", base="WORKING", remote="HEAD"
         )
 
         # Then
         mock_execute.assert_called_once_with(
-            ["git", "diff", "HEAD", "--name-only", "-z"], cwd="/bin/test-path"
+            ["git", "diff", "HEAD", "--name-only", "-z"],
+            cwd="test-path",
         )
         assert {"code": 0, "files": ["file1.ipynb", "file2.py"]} == actual_response
 
@@ -70,14 +73,14 @@ async def test_changed_files_index():
         mock_execute.return_value = maybe_future((0, "file1.ipynb\x00file2.py", ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).changed_files(
-            current_path="test-path", base="INDEX", remote="HEAD"
+        actual_response = await Git().changed_files(
+            path="test-path", base="INDEX", remote="HEAD"
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "diff", "--staged", "HEAD", "--name-only", "-z"],
-            cwd="/bin/test-path",
+            cwd="test-path",
         )
         assert {"code": 0, "files": ["file1.ipynb", "file2.py"]} == actual_response
 
@@ -89,14 +92,14 @@ async def test_changed_files_two_commits():
         mock_execute.return_value = maybe_future((0, "file1.ipynb\x00file2.py", ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).changed_files(
-            current_path="test-path", base="HEAD", remote="origin/HEAD"
+        actual_response = await Git().changed_files(
+            path="test-path", base="HEAD", remote="origin/HEAD"
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "diff", "HEAD", "origin/HEAD", "--name-only", "-z"],
-            cwd="/bin/test-path",
+            cwd="test-path",
         )
         assert {"code": 0, "files": ["file1.ipynb", "file2.py"]} == actual_response
 
@@ -108,14 +111,14 @@ async def test_changed_files_git_diff_error():
         mock_execute.side_effect = CalledProcessError(128, b"cmd", b"error message")
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).changed_files(
-            current_path="test-path", base="HEAD", remote="origin/HEAD"
+        actual_response = await Git().changed_files(
+            path="test-path", base="HEAD", remote="origin/HEAD"
         )
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "diff", "HEAD", "origin/HEAD", "--name-only", "-z"],
-            cwd="/bin/test-path",
+            cwd="test-path",
         )
         assert {"code": 128, "message": "error message"} == actual_response
 
@@ -221,12 +224,54 @@ async def test_is_binary_file(args, cli_result, cmd, expected):
 
         if isinstance(expected, type) and issubclass(expected, Exception):
             with pytest.raises(expected):
-                await Git(FakeContentManager("/bin"))._is_binary(*args)
+                await Git()._is_binary(*args)
         else:
             # When
-            actual_response = await Git(FakeContentManager("/bin"))._is_binary(*args)
+            actual_response = await Git()._is_binary(*args)
 
             # Then
             mock_execute.assert_called_once_with(cmd, cwd="/bin")
 
             assert actual_response == expected
+
+
+@pytest.mark.asyncio
+async def test_Git_get_nbdiff_file():
+    HERE = Path(__file__).parent.resolve()
+
+    manager = Git()
+    prev_content = (HERE / "samples" / "ipynb_base.json").read_text()
+    curr_content = (HERE / "samples" / "ipynb_remote.json").read_text()
+
+    result = await manager.get_nbdiff(prev_content, curr_content)
+
+    expected_result = json.loads((HERE / "samples" / "ipynb_nbdiff.json").read_text())
+    assert result == expected_result
+
+
+@pytest.mark.asyncio
+async def test_Git_get_nbdiff_dict():
+    HERE = Path(__file__).parent.resolve()
+
+    manager = Git()
+    prev_content = json.loads((HERE / "samples" / "ipynb_base.json").read_text())
+    curr_content = json.loads((HERE / "samples" / "ipynb_remote.json").read_text())
+
+    result = await manager.get_nbdiff(prev_content, curr_content)
+
+    expected_result = json.loads((HERE / "samples" / "ipynb_nbdiff.json").read_text())
+    assert result == expected_result
+
+
+@pytest.mark.asyncio
+async def test_Git_get_nbdiff_no_content():
+    HERE = Path(__file__).parent.resolve()
+
+    manager = Git()
+
+    result = await manager.get_nbdiff("", "")
+
+    assert result == {
+        "base": nbformat.versions[nbformat.current_nbformat].new_notebook(),
+        "diff": [],
+    }
