@@ -1,14 +1,11 @@
-import os
-from subprocess import CalledProcessError
-from unittest.mock import Mock, call, patch
+from unittest.mock import patch
 
 import pytest
-import tornado
 
 from jupyterlab_git import JupyterLabGit
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, maybe_future
+from .testutils import maybe_future
 
 
 @pytest.mark.asyncio
@@ -18,11 +15,9 @@ async def test_init():
         mock_execute.return_value = maybe_future((0, "", ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).init("test_curr_path")
+        actual_response = await Git().init("test_curr_path")
 
-        mock_execute.assert_called_once_with(
-            ["git", "init"], cwd=os.path.join("/bin", "test_curr_path")
-        )
+        mock_execute.assert_called_once_with(["git", "init"], cwd="test_curr_path")
 
         assert {"code": 0, "actions": None} == actual_response
 
@@ -38,13 +33,10 @@ async def test_init_and_post_init():
 
         # When
         actual_response = await Git(
-            FakeContentManager("/bin"),
             JupyterLabGit(actions={"post_init": ['echo "hello"']}),
         ).init("test_curr_path")
 
-        mock_execute.assert_called_with(
-            ["echo", "hello"], cwd=os.path.join("/bin", "test_curr_path")
-        )
+        mock_execute.assert_called_with(["echo", "hello"], cwd="test_curr_path")
 
         assert {
             "code": 0,
@@ -65,13 +57,10 @@ async def test_init_and_post_init_fail():
 
         # When
         actual_response = await Git(
-            FakeContentManager("/bin"),
             JupyterLabGit(actions={"post_init": ["not_there arg"]}),
         ).init("test_curr_path")
 
-        mock_execute.assert_called_with(
-            ["not_there", "arg"], cwd=os.path.join("/bin", "test_curr_path")
-        )
+        mock_execute.assert_called_with(["not_there", "arg"], cwd="test_curr_path")
 
         assert {
             "code": 1,
@@ -99,13 +88,10 @@ async def test_init_and_post_init_fail_to_run():
 
         # When
         actual_response = await Git(
-            FakeContentManager("/bin"),
             JupyterLabGit(actions={"post_init": ["not_there arg"]}),
         ).init("test_curr_path")
 
-        mock_execute.assert_called_with(
-            ["not_there", "arg"], cwd=os.path.join("/bin", "test_curr_path")
-        )
+        mock_execute.assert_called_with(["not_there", "arg"], cwd="test_curr_path")
 
         assert {
             "code": 1,
