@@ -1,12 +1,10 @@
-import os
-from unittest.mock import Mock, call, patch
+from unittest.mock import call, patch
 
 import pytest
-import tornado
 
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, maybe_future
+from .testutils import maybe_future
 
 
 @pytest.mark.asyncio
@@ -19,14 +17,12 @@ async def test_git_pull_fail():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).pull(
-                "test_curr_path"
-            )
+            actual_response = await Git().pull("test_curr_path")
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "pull", "--no-commit"],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
             )
             assert {"code": 1, "message": "Authentication failed"} == actual_response
@@ -46,16 +42,14 @@ async def test_git_pull_with_conflict_fail():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).pull(
-                "test_curr_path"
-            )
+            actual_response = await Git().pull("test_curr_path")
 
             # Then
             mock_execute.assert_has_calls(
                 [
                     call(
                         ["git", "pull", "--no-commit"],
-                        cwd="/bin/test_curr_path",
+                        cwd="test_curr_path",
                         env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
                     )
                 ]
@@ -81,16 +75,14 @@ async def test_git_pull_with_auth_fail():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).pull(
-                "test_curr_path", auth
-            )
+            actual_response = await Git().pull("test_curr_path", auth)
 
             # Then
             mock_execute_with_authentication.assert_called_once_with(
                 ["git", "pull", "--no-commit"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {
@@ -109,14 +101,12 @@ async def test_git_pull_success():
             mock_execute.return_value = maybe_future((0, output, ""))
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).pull(
-                "test_curr_path"
-            )
+            actual_response = await Git().pull("test_curr_path")
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "pull", "--no-commit"],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
             )
             assert {"code": 0, "message": output} == actual_response
@@ -135,16 +125,14 @@ async def test_git_pull_with_auth_success():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).pull(
-                "test_curr_path", auth
-            )
+            actual_response = await Git().pull("test_curr_path", auth)
 
             # Then
             mock_execute_with_authentication.assert_called_once_with(
                 ["git", "pull", "--no-commit"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {"code": 0, "message": output} == actual_response
@@ -165,16 +153,14 @@ async def test_git_pull_with_auth_success_and_conflict_fail():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).pull(
-                "test_curr_path", auth
-            )
+            actual_response = await Git().pull("test_curr_path", auth)
 
             # Then
             mock_execute_with_authentication.assert_has_calls(
                 [
                     call(
                         ["git", "pull", "--no-commit"],
-                        cwd="/bin/test_curr_path",
+                        cwd="test_curr_path",
                         env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
                         username="asdf",
                         password="qwerty",
@@ -197,14 +183,14 @@ async def test_git_push_fail():
             )
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).push(
+            actual_response = await Git().push(
                 "test_origin", "HEAD:test_master", "test_curr_path"
             )
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "push", "test_origin", "HEAD:test_master"],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
             )
             assert {"code": 1, "message": "Authentication failed"} == actual_response
@@ -226,7 +212,7 @@ async def test_git_push_with_auth_fail():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).push(
+            actual_response = await Git().push(
                 "test_origin", "HEAD:test_master", "test_curr_path", auth
             )
 
@@ -235,7 +221,7 @@ async def test_git_push_with_auth_fail():
                 ["git", "push", "test_origin", "HEAD:test_master"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {
@@ -254,14 +240,14 @@ async def test_git_push_success():
             mock_execute.return_value = maybe_future((0, output, "does not matter"))
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).push(
+            actual_response = await Git().push(
                 ".", "HEAD:test_master", "test_curr_path"
             )
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "push", ".", "HEAD:test_master"],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "0"},
             )
             assert {"code": 0, "message": output} == actual_response
@@ -280,7 +266,7 @@ async def test_git_push_with_auth_success():
 
             # When
             auth = {"username": "asdf", "password": "qwerty"}
-            actual_response = await Git(FakeContentManager("/bin")).push(
+            actual_response = await Git().push(
                 ".", "HEAD:test_master", "test_curr_path", auth
             )
 
@@ -289,7 +275,7 @@ async def test_git_push_with_auth_success():
                 ["git", "push", ".", "HEAD:test_master"],
                 username="asdf",
                 password="qwerty",
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
                 env={"TEST": "test", "GIT_TERMINAL_PROMPT": "1"},
             )
             assert {"code": 0, "message": output} == actual_response
