@@ -1,12 +1,10 @@
-import os
-from unittest.mock import Mock, call, patch
+from unittest.mock import patch
 
 import pytest
-import tornado
 
 from jupyterlab_git.git import Git
 
-from .testutils import FakeContentManager, ServerTest, maybe_future
+from .testutils import maybe_future
 
 
 @pytest.mark.asyncio
@@ -17,12 +15,12 @@ async def test_git_tag_success():
         mock_execute.return_value = maybe_future((0, tag, ""))
 
         # When
-        actual_response = await Git(FakeContentManager("/bin")).tags("test_curr_path")
+        actual_response = await Git().tags("test_curr_path")
 
         # Then
         mock_execute.assert_called_once_with(
             ["git", "tag", "--list"],
-            cwd=os.path.join("/bin", "test_curr_path"),
+            cwd="test_curr_path",
         )
 
         assert {"code": 0, "tags": [tag]} == actual_response
@@ -37,14 +35,12 @@ async def test_git_tag_checkout_success():
             mock_execute.return_value = maybe_future((0, "", ""))
 
             # When
-            actual_response = await Git(FakeContentManager("/bin")).tag_checkout(
-                "test_curr_path", "mock_tag"
-            )
+            actual_response = await Git().tag_checkout("test_curr_path", "mock_tag")
 
             # Then
             mock_execute.assert_called_once_with(
                 ["git", "checkout", "tags/{}".format(tag)],
-                cwd=os.path.join("/bin", "test_curr_path"),
+                cwd="test_curr_path",
             )
 
             assert {
